import constructive
import refinement
import time
import random
import numpy as np

def multiStart(objects, capacity, nItersMH, timeLimitMH):
    SBest = [0 for i in objects]
    SBestBenefit = 0
    SBestWeight = 0
    iter = 0
    startTime = time.time()
    while(iter < nItersMH and time.time() - startTime < timeLimitMH):
        iter += 1
        [S, SBenefit, SWeight] = constructive.partiallyGreedyHeuristic(objects, capacity, 1.0)
        print("Iter:%d - %f" % (iter, SBenefit))
        [S, SBenefit, SWeight] = refinement.firstImprovementAscendentMethod(S, SBenefit, SWeight, objects, capacity)
        print("Iter:%d - %f" % (iter, SBenefit))
        if(SBenefit > SBestBenefit):
            SBest = S.copy()
            SBestBenefit = SBenefit
            SBestWeight = SWeight
            print("MS: " + str(SBestBenefit))
    S = SBest.copy()
    SBenefit = SBestBenefit
    SWeight = SBestWeight
    return [S, SBenefit, SWeight]

def grasp(objects, capacity, alpha, nItersMH, timeLimitMH):
    SBest = [0 for i in objects]
    SBestBenefit = 0
    SBestWeight = 0
    iter = 0
    startTime = time.time()
    while(iter < nItersMH and time.time() - startTime < timeLimitMH):
        iter += 1
        [S, SBenefit, SWeight] = constructive.partiallyGreedyHeuristic(objects, capacity, alpha)
        print("Iter:%d - %f" % (iter, SBenefit))
        [S, SBenefit, SWeight] = refinement.firstImprovementAscendentMethod(S, SBenefit, SWeight, objects, capacity)
        print("Iter:%d - %f" % (iter, SBenefit))
        if(SBenefit > SBestBenefit):
            SBest = S.copy()
            SBestBenefit = SBenefit
            SBestWeight = SWeight
            print("GRASP: " + str(SBestBenefit))
    S = SBest.copy()
    SBenefit = SBestBenefit
    SWeight = SBestWeight
    return [S, SBenefit, SWeight]

def tabuSearch(S, SBenefit, SWeight, objects, capacity, BTmax, Tsize):
    SBest = S.copy()
    SBestBenefit = SBenefit
    SBestWeight = SWeight
    iter = 0
    melhorIter = 0
    T = []
    while(iter - melhorIter <= BTmax):
        iter += 1
        [SCand, SCandBenefit, SCandWeight, i, j] = refinement.tabuSearchNeighborhood(S, SBenefit, SWeight, objects, capacity, T, SBestBenefit)
        if(len(T) < Tsize):
            T.append([i, j])
        else:
            T.pop(0)
            T.append([i, j])
        print(T)
        S = SCand.copy()
        SBenefit = SCandBenefit
        SWeight = SCandWeight
        if(SBenefit > SBestBenefit):
            SBest = S.copy()
            SBestBenefit = SBenefit
            SBestWeight = SWeight
            print(SBestBenefit)
            melhorIter = iter
    S = SBest.copy()
    SBenefit = SBestBenefit
    SWeight = SBestWeight
    return [S, SBenefit, SWeight]

def simulatedAnnealing(S, SBenefit, SWeight, objects, capacity, alpha, SAmax, T0):
    SBest = S.copy()
    SBestBenefit = SBenefit
    SBestWeight = SWeight
    iterT = 0
    T = T0
    while(T > 0.00001):
        while(iterT < SAmax):
            iterT += 1
            i = random.randint(0, len(objects) - 1)
#            j = random.randint(0, len(objects) - 1)
            SCand = S.copy()
            SCandWeight = SWeight
            SCandBenefit = SBenefit
            #Switch i bit
            if(SCand[i] == 1):
                SCand[i] = 0
                SCandWeight -= objects[i].weight
                SCandBenefit -= objects[i].benefit
            else:
                SCand[i] = 1
                SCandWeight += objects[i].weight
                SCandBenefit += objects[i].benefit
            #Switch j bit
#            if(SCand[j] == 1):
#                SCand[j] = 0
#                SCandWeight -= objects[j].weight
#                SCandBenefit -= objects[j].benefit
#            else:
#                SCand[j] = 1
#                SCandWeight += objects[j].weight
#                SCandBenefit += objects[j].benefit
            if(SCandWeight <= capacity):
                delta = SCandBenefit - SBenefit
                if(delta > 0):
                    S = SCand.copy()
                    SBenefit = SCandBenefit
                    SWeight = SCandWeight
                    if(SBenefit > SBestBenefit):
                        SBest = S.copy()
                        SBestBenefit = SBenefit
                        SBestWeight = SWeight
                        print(SBestBenefit)
                else:
                    x = random.random()
                    if(x <  np.exp(delta/T)):
                        S = SCand.copy()
                        SBenefit = SCandBenefit
                        SWeight = SCandWeight
        T = alpha * T
        print(str(T) + " " + str(SBenefit))
        iterT = 0
    S = SBest.copy()
    SBenefit = SBestBenefit
    SWeight = SBestWeight
    return [S, SBenefit, SWeight]

##def setT0(S, SBenefit, SWeight, objects, capacity, alpha, SAmax, T0):
##    SBest = S.copy()
##    SBestBenefit = SBenefit
##    SBestWeight = SWeight
    

#def vns(S, SBenefit, SWeight, objects, capacity, nItersMH, timeLimitMH):
    
