
import constructive
import refinement
import metaheuristics

#Object class
class Object:
    def __init__(self, id, weight, benefit):
        self.id = int(id)
        self.weight = int(weight)
        self.benefit = int(benefit)
        self.bw = float(benefit) / float(weight)

    def __eq__(self, other):
        return self.id == other.id and self.weight == other.weight and self.benefit == other.benefit

    def __lt__(self, other):
        return self.bw > other.bw

    def __str__(self):
        return str(self.id) + " " + str(self.weight) + " " + str(self.benefit) + " " + str(self.bw)
        
#Program parameters
fileName = "Instances/knapdata2.txt"

#File reading
file = open(fileName)
capacity = 0
numObjects = 0
capacity = int(file.readline())
numObjects = int(file.readline())
objects = []
for line in file:
    lineObj = line.split(" ")
    id = int(lineObj[0])
    weight = int(lineObj[1])
    benefit = int(lineObj[2])
    obj = Object(id, weight, benefit)
    objects.append(obj)
print("Capacity: " + str(capacity))
print("Num objects: " + str(numObjects))
#print("Objects sorted by benefit / weight: ")
#for obj in objectsSorted:
#    print(obj)

constructiveHeuristic = 1
alpha = 0.01
refinementHeuristic = 10
nItersRefinement = 10000
timeLimitRefinement = 60000
nItersMH = 3000
timeLimitMH = 60
BTmax = 100
Tsize = 5
SAalpha = 0.8
k = 10
SAmax = k * numObjects
T0 = 500

print("\nSolving...")

#Constructive heuristic
if(constructiveHeuristic == 1):
    [S, SBenefit, SWeight] = constructive.greedyHeuristic(objects, capacity)
elif(constructiveHeuristic == 2):
    [S, SBenefit, SWeight] = constructive.partiallyGreedyHeuristic(objects, capacity, alpha)

print("\nS = " + str(S))
print("Weight = " + str(SWeight))
print("Benefit = " + str(SBenefit))    
    
#Refinement Heuristic
if(refinementHeuristic == 1):
    [S, SBenefit, SWeight] = refinement.ascendentMethod(S, SBenefit, SWeight, objects, capacity)
elif(refinementHeuristic == 2):
    [S, SBenefit, SWeight] = refinement.firstImprovementAscendentMethod(S, SBenefit, SWeight, objects, capacity)
elif(refinementHeuristic == 3):
    [S, SBenefit, SWeight] = refinement.randomAscendentMethod(S, SBenefit, SWeight, objects, capacity, nItersRefinement, timeLimitRefinement)
elif(refinementHeuristic == 4):
    [S, SBenefit, SWeight] = refinement.randomNotDescendentMethod(S, SBenefit, SWeight, objects, capacity, nItersRefinement, timeLimitRefinement)
elif(refinementHeuristic == 5):
    [S, SBenefit, SWeight] = metaheuristics.multiStart(objects, capacity, nItersMH, timeLimitMH)
elif(refinementHeuristic == 6):
    [S, SBenefit, SWeight] = metaheuristics.grasp(objects, capacity, alpha, nItersMH, timeLimitMH)
elif(refinementHeuristic == 7):
    [S, SBenefit, SWeight] = constructive.greedyHeuristic(objects, capacity)
    [S, SBenefit, SWeight] = metaheuristics.tabuSearch(S, SBenefit, SWeight, objects, capacity, BTmax, Tsize)
elif(refinementHeuristic == 8):
    [S, SBenefit, SWeight] = constructive.greedyHeuristic(objects, capacity)
    [S, SBenefit, SWeight] = metaheuristics.simulatedAnnealing(S, SBenefit, SWeight, objects, capacity, SAalpha, SAmax, T0)
elif(refinementHeuristic == 9):
    [S, SBenefit, SWeight] = constructive.greedyHeuristic(objects, capacity)
    [S, SBenefit, SWeight] = metaheuristics.vnd(S, SBenefit, SWeight, objects, capacity)
elif(refinementHeuristic == 10):
    [S, SBenefit, SWeight] = constructive.greedyHeuristic(objects, capacity)
    [S, SBenefit, SWeight] = metaheuristics.ils(S, SBenefit, SWeight, objects, capacity, nItersMH, timeLimitMH)
        
#Writing output
print("\nS = " + str(S))
print("Weight = " + str(SWeight))
print("Benefit = " + str(SBenefit))
file = open(fileName + ".sol", "w")
file.write(str(SBenefit) + "\n")
for s in S:
    file.write(str(s) + " ")
file.close()

